//+------------------------------------------------------------------+
//|                                                SAR Color Bar.mq4 |
//+------------------------------------------------------------------+
#property copyright "Kalenzo Code adapted by cja"
#property link      "http://www.foreksik.prv.pl"

#property indicator_color1 Red
#property indicator_color2 Green
#property indicator_width1 3
#property indicator_width2 3
#property indicator_separate_window
#property indicator_buffers 2
#property indicator_maximum 1
#property indicator_minimum 0

//
//
//
//
//

extern string TimeFrame          = "Current time frame";
extern double Step               = 0.013;
extern double Maximum            = 0.2;
extern int    Precision          = 5;
extern bool   alertsOn           = false;
extern bool   alertsOnCurrent    = false;
extern bool   alertsMessage      = true;
extern bool   alertsNotification = false;
extern bool   alertsSound        = false;
extern bool   alertsEmail        = false;
extern bool   alertsNotify       = false;

double UpH[];
double DnH[];  
double trend[];

//
//
//
//
//

string indicatorFileName;
int    timeFrame;
bool   returnBars;
bool   calculateValue;

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
  {
//---- indicators
   IndicatorBuffers(3);
   SetIndexBuffer(0,UpH); SetIndexStyle(0,DRAW_HISTOGRAM); SetIndexLabel(0,"SAR Up Channel");
   SetIndexBuffer(1,DnH); SetIndexStyle(1,DRAW_HISTOGRAM); SetIndexLabel(1,"SAR Down Channel");  
   SetIndexBuffer(2,trend);
  
      //
      //
      //
      //
      //
     
        indicatorFileName = WindowExpertName();
        calculateValue    = (TimeFrame=="calculateValue"); if (calculateValue) return(0);
        returnBars        = (TimeFrame=="returnBars");     if (returnBars)     return(0);
        timeFrame         = stringToTimeFrame(TimeFrame); 
        
      //
      //
      //
      //
      //
     
     IndicatorShortName(timeFrameToString(timeFrame)+"  Sar Colored Bar");
     
return(0);
}  
     
//+------------------------------------------------------------------+
//| Custor indicator deinitialization function                       |
//+------------------------------------------------------------------+
int deinit()
  {
//---- 
   
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+

int start()
{
   int counted_bars=IndicatorCounted();
   int i,limit;
   
   
   if(counted_bars < 0) return(-1);
   if(counted_bars > 0) counted_bars--;
           limit = MathMin(Bars-counted_bars,Bars-1); 
           if (returnBars) { UpH[0] = limit+1; return(0); }

   //
   //
   //
   //
   //

   if (calculateValue || timeFrame==Period())
   {
     for(i=limit; i>=0; i--)
     {
        double sar = NormalizeDouble(iSAR(NULL,0,Step,Maximum,i),Precision);
        UpH[i] = EMPTY_VALUE;
        DnH[i] = EMPTY_VALUE;
        trend[i] = trend[i+1];
        if(sar >= iHigh(NULL,0,i)) trend[i] = 1;
        else                       trend[i] =-1;
        if(trend[i] == 1) UpH[i] = 1;
        if(trend[i] ==-1) DnH[i] = 1;
     }
     manageAlerts();
     return(0);
    }
    
    //
    //
    //
    //
    //
   
    limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,timeFrame,indicatorFileName,"returnBars",0,0)*timeFrame/Period()));
    for (i=limit;i>=0; i--)
    {
       int y = iBarShift(NULL,timeFrame,Time[i]);
          trend[i] = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",Step,Maximum,Precision,2,y);
          UpH[i]   = EMPTY_VALUE;
          DnH[i]   = EMPTY_VALUE; 
          if (trend[i] == 1) UpH[i] = 1;
          if (trend[i] ==-1) DnH[i] = 1;   
    }
    manageAlerts();  
return(0);         
}


//+-------------------------------------------------------------------
//|                                                                  
//+-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = stringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}
string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string stringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int tchar = StringGetChar(s, length);
         if((tchar > 96 && tchar < 123) || (tchar > 223 && tchar < 256))
                     s = StringSetChar(s, length, tchar - 32);
         else if(tchar > -33 && tchar < 0)
                     s = StringSetChar(s, length, tchar + 224);
   }
   return(s);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

void manageAlerts()
{
   if (!calculateValue && alertsOn)
   {
      if (alertsOnCurrent)
           int whichBar = 0;
      else     whichBar = 1; whichBar = iBarShift(NULL,0,iTime(NULL,timeFrame,whichBar));
      if (trend[whichBar] != trend[whichBar+1])
      {
         if (trend[whichBar] == 1)  doAlert(whichBar,"Buy");
         if (trend[whichBar] ==-1)  doAlert(whichBar,"Sell");
      }         
   }
}   

//
//
//
//
//

void doAlert(int forBar, string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
      if (previousAlert != doWhat || previousTime != Time[forBar]) {
          previousAlert  = doWhat;
          previousTime   = Time[forBar];

          //
          //
          //
          //
          //

          message =  StringConcatenate(Symbol()," ",timeFrameToString(timeFrame)," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," SAR Color Bar ",doWhat);
             if (alertsMessage) Alert(message);
             if (alertsNotify)  SendNotification(StringConcatenate(Symbol(), Period() ," SAR Color Bar " +" "+message));
             if (alertsEmail)   SendMail(StringConcatenate(Symbol()," SAR Color Bar "),message);
             if (alertsSound)   PlaySound("alert2.wav");
      }
}

